@@ -8,8 +8,7 @@
 ## smooth = parameter 'h' from AGM
 
 bsmc2.internal <- function (object, params, Np, est,
-  smooth, tol, seed = NULL,
-  verbose = getOption("verbose"),
+  smooth, tol, verbose = getOption("verbose"),
   max.fail, transform, .getnativesymbolinfo = TRUE,
   ...) {
 
@@ -22,18 +21,9 @@ bsmc2.internal <- function (object, params, Np, est,
   ptsi.inv <- ptsi.for <- TRUE
   transform <- as.logical(transform)
 
-  if (!is.null(seed))
-    warning(ep,"argument ",sQuote("seed"),
-      " now has no effect.  Consider using ",
-      sQuote("freeze"),".",call.=FALSE)
-
-  if (missing(params)) {
-    if (length(coef(object))>0) {
-      params <- coef(object)
-    } else {
-      stop(ep,sQuote("params")," must be supplied",call.=FALSE)
-    }
-  }
+  if (missing(params)) params <- coef(object)
+  if (is.list(params)) params <- unlist(params)
+  if (length(params)==0) stop(ep,sQuote("params")," must be supplied",call.=FALSE)
 
   if (missing(Np)) Np <- NCOL(params)
   else if (is.matrix(params)&&(Np!=ncol(params))) {
